const getNestedValue = function (data, keyArr) {
  let currentIndex = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  if (currentIndex === keyArr.length) {
    if (typeof data === 'string') {
      return data;
    }
    return null;
  }
  if (typeof data !== 'string') {
    return getNestedValue(data?.[keyArr?.[currentIndex]], keyArr, currentIndex + 1);
  }
  return null;
};
export const getI18nValue = (data, key, getDefaultText) => {
  const keyArr = key.split('.');
  const text = getNestedValue(data, keyArr);
  if (!text && getDefaultText) {
    return getDefaultText(key);
  }
  return text || key;
};
export default function createI18n() {
  let data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  let currentLang;
  let listenHandlers = [];
  if (data === null || typeof data !== 'object') {
    throw new Error('React Simple I18n: Language data should be an object');
  }
  if (options === null || typeof options !== 'object') {
    throw new Error('React Simple I18n: Options should be an object');
  }
  const {
    lang,
    getDefaultText
  } = options;
  const langData = data;
  currentLang = lang || (data.enUS ? 'enUS' : null);
  function t(key) {
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }
    if (!currentLang) {
      throw new Error('React Simple I18n: Current language must be set before using t()');
    }
    if (!key || typeof key !== 'string') {
      throw new Error('React Simple I18n: t() must have string type as its first argument');
    }
    const value = getI18nValue(langData[currentLang], key, getDefaultText);
    if (args.length === 0) {
      return value;
    } else {
      let i = 0;
      return value.replace(/%s%/gi, () => {
        const output = args[i];
        i += 1;
        return typeof output !== 'undefined' ? output : '%s%';
      });
    }
  }
  function listen(handler) {
    listenHandlers.push(handler);
  }
  function unlisten(handler) {
    listenHandlers = listenHandlers.filter(func => handler !== func);
  }
  function getLang() {
    return currentLang;
  }
  function setLang(lang) {
    if (lang === currentLang) return;
    if (!data[lang]) {
      return console.error(`Language '${lang}' is not found.`);
    }
    currentLang = lang;
    listenHandlers.forEach(func => {
      func({});
    });
  }
  function addLangData(data) {
    Object.assign(langData, data);
  }
  function _getListenHandlers() {
    return listenHandlers;
  }
  return {
    t,
    getLang,
    setLang,
    listen,
    unlisten,
    addLangData,
    _getListenHandlers
  };
}